// Copyright (C) 1999 Jean-Marc Valin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//#define _PLUGIN_NODE

#include "FrameAccumulator.h"
#include "FrameOperation.h"
#include "GrowingFrameBuffer.h"
#include "Network.h"
#include "multithread.h"

//static int tata = Network::addFactory ("ACCUMULATOR", new NodeFactory<FrameAccumulator>);
//DECLARE_NODE(FrameAccumulator)
NODE_INFO(FrameAccumulator, "Signal:Manip", "INPUT", "OUTPUT", "LENGTH:CACHEALL")

FrameAccumulator::FrameAccumulator(string nodeName, const ParameterSet &params) 
   : FrameOperation(nodeName, params) 
   , currentPos(0)
{
   try {
      inputID = addInput("INPUT");
      if (parameters.exist("INPUTLENGTH"))
         inputLength = dereference_cast<int> (parameters.get("INPUTLENGTH"));
      else inputLength = dereference_cast<int> (parameters.get("LENGTH"));
   } catch (BaseException *e) 
   {
      //e->print();
      throw e->add(new NodeException (NULL,"Caught Exception in FrameAccumulator constructor",__FILE__,__LINE__));
   }
}

void FrameAccumulator::specificInitialize()
{
   //cerr << "FrameAccumulator initialize...\n";
   this->FrameOperation::specificInitialize();
   currentPos=0;
}

void FrameAccumulator::reset()
{
   //cerr << "FrameAccumulator initialize...\n";
   this->FrameOperation::reset();
   currentPos=0;
}

ObjectRef FrameAccumulator::getOutput(int output_id, int count)
{
   int i, j;
   lock();
   //cerr << "FrameAccumulator::getOutput for " << name << " count = " << count << endl;
   for (i=processCount+1;i<=count;i++)
   {
      NodeInput input = inputs[inputID];
      ObjectRef inputResult;

      inputResult = input.node->getOutput(input.outputID, i);

      if (!inputResult->status)
      {
         const Vector<float> &inputFrame = object_cast<Vector<float> > (inputResult);

         Buffer &out = object_cast<Buffer> (output);     
         Vector<float> &outputFrame = object_cast<Vector<float> > (out[currentPos]);

         for (j=0;j<outputLength;j++)
            outputFrame[j]=inputFrame[j];
         currentPos++;
      }
      //else
       //  cerr << "accumulator: nil output\n";
      //  return inputResult;  //propagete the "no output" result
      processCount = i;
      
   }
   return unlock_and_return(output);
}
