// Copyright (C) 1999 Jean-Marc Valin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stream.h>
#include "BufferedNode.h"
#include "Buffer.h"
#include "Vector.h"
#include "Exception.h"
#include "GrowingFrameBuffer.h"
#include "RotatingFrameBuffer.h"
class PCMConvert;

//DECLARE_NODE(PCMConvert)
NODE_INFO(PCMConvert,"Signal:Audio", "INPUT", "OUTPUT", "LENGTH:INPUTFORMAT:OUTPUTFORMAT")

#define CLIP 32635
#define BIAS 0
static unsigned char linear2mulaw(short sample)
{
    static int exp_lut[256] = {0,0,1,1,2,2,2,2,3,3,3,3,3,3,3,3,
                                 4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
                                 5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
                                 5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
                                 6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                                 6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                                 6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                                 6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                                 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                                 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                                 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                                 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                                 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                                 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                                 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                                 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7};
    int sign, exponent, mantissa;
    unsigned char ulawbyte;
    
    /* Get the sample into sign-magnitude. */
    sign = (sample >> 8) & 0x80;                /* set aside the sign */
    if ( sign != 0 ) sample = -sample;          /* get magnitude */
    if ( sample > CLIP ) sample = CLIP;         /* clip the magnitude */
    
    /* Convert from 16 bit linear to ulaw. */
    sample = sample + BIAS;
    exponent = exp_lut[( sample >> 7 ) & 0xFF];
    mantissa = ( sample >> ( exponent + 3 ) ) & 0x0F;
    /*Bug kludge here*/
    if (sample == 0)
      sign=0x80;
    ulawbyte = ~ ( sign | ( exponent << 4 ) | mantissa );
    return ulawbyte;
}



class PCMConvert : public BufferedNode {
   
   enum encoding {ulaw, alaw, lin16, lin8};
   int inputID;
   int outputID;
   int inputLength;
   int outputLength;
   encoding outputEncoding;
   void (*toFloat) (void *, float *, int);
   void (*fromFloat) (float *, void *, int);
   
public:

   PCMConvert(string nodeName, ParameterSet params)
   : BufferedNode(nodeName, params)
   {
      inputID = addInput("INPUT");

      outputID = addOutput("OUTPUT");
      if (parameters.exist("OUTPUTLENGTH"))
         outputLength = dereference_cast <int> (parameters.get("OUTPUTLENGTH"));
      else outputLength = dereference_cast <int> (parameters.get("LENGTH"));
      if (parameters.exist("CACHEALL"))
         outputs[outputID].cacheAll=true;
      if (parameters.exist("INPUTLENGTH"))
         inputLength = dereference_cast<int> (parameters.get("INPUTLENGTH"));
      else inputLength = dereference_cast<int> (parameters.get("LENGTH"));

      String inputFormat = object_cast<String> (parameters.get("INPUTFORMAT"));
      if (inputFormat != "FLOAT")
         throw new NodeException (NULL, "Only FLOAT supported as input Format in PCMConvert", __FILE__, __LINE__);

      String outputFormat = object_cast<String> (parameters.get("OUTPUTFORMAT"));
      if (outputFormat == "ULAW")
         outputEncoding=ulaw;
      else if (outputFormat == "LIN16")
         outputEncoding=lin16;
      else throw new NodeException (NULL, "Only ULAW and LIN16 supported as output Format in PCMConvert", __FILE__, __LINE__);
   }

   ~PCMConvert() {}

   virtual void specificInitialize()
   {
      this->BufferedNode::specificInitialize();
      if (outputEncoding == ulaw)
      {

         if (outputs[outputID].cacheAll)
         {
            outputs[outputID].buffer = ObjectRef(new GrowingFrameBuffer<unsigned char> (outputLength, outputs[outputID].lookAhead+outputs[outputID].lookBack+1));
         } else {
            outputs[outputID].buffer = ObjectRef(new RotatingFrameBuffer<unsigned char> (outputLength, outputs[outputID].lookAhead+outputs[outputID].lookBack+1));
         }

      } else {

         if (outputs[outputID].cacheAll)
         {
            outputs[outputID].buffer = ObjectRef(new GrowingFrameBuffer<short> (outputLength, outputs[outputID].lookAhead+outputs[outputID].lookBack+1));
         } else {
            outputs[outputID].buffer = ObjectRef(new RotatingFrameBuffer<short> (outputLength, outputs[outputID].lookAhead+outputs[outputID].lookBack+1));
         }      

      }
   }

   virtual void reset()
   {
      this->BufferedNode::specificInitialize();
      if (outputEncoding == ulaw)
      {

         if (outputs[outputID].cacheAll)
         {
            outputs[outputID].buffer = ObjectRef(new GrowingFrameBuffer<unsigned char> (outputLength, outputs[outputID].lookAhead+outputs[outputID].lookBack+1));
         } else {
            outputs[outputID].buffer = ObjectRef(new RotatingFrameBuffer<unsigned char> (outputLength, outputs[outputID].lookAhead+outputs[outputID].lookBack+1));
         }

      } else {

         if (outputs[outputID].cacheAll)
         {
            outputs[outputID].buffer = ObjectRef(new GrowingFrameBuffer<short> (outputLength, outputs[outputID].lookAhead+outputs[outputID].lookBack+1));
         } else {
            outputs[outputID].buffer = ObjectRef(new RotatingFrameBuffer<short> (outputLength, outputs[outputID].lookAhead+outputs[outputID].lookBack+1));
         }      

      }

   }

   void calculate(int output_id, int count, Buffer &out)
   {
      NodeInput input = inputs[inputID];
      ObjectRef inputValue = input.node->getOutput(input.outputID, count);

      if (inputValue->status != Object::valid)
      {
         out[count]->status = inputValue->status;
         return;
      }
      const Vector<float> &in = object_cast<Vector<float> > (inputValue);
      
      if (outputEncoding == ulaw)
      {
         Vector<unsigned char> &output = object_cast<Vector<unsigned char> > (out[count]);
         for (int i=0;i<outputLength;i++)
         {
            output[i]=linear2mulaw(short(in[i]));
         }
         output.status = Object::valid;
      } else if (outputEncoding == lin16)
      {
         Vector<short> &output = object_cast<Vector<short> > (out[count]);
         for (int i=0;i<outputLength;i++)
         {
            output[i]=short(in[i]);
         }
         output.status = Object::valid;
      }
      
   }

};
