// Copyright (C) 1999 Dominic Letourneau and Andre Charbonneau
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/*
** Includes
*/
#include "FileParserException.h"

FileParserException::FileParserException(int type, const string& description)
{
  m_type = type;
  m_description = description;
  return;
}

void FileParserException::print(ostream &out)
{
  out << "FileParserException: ";

  switch(this->get_type())
    {
    case NO_TYPE:
      {
	out << "type not set";
	break;
      }
    case FATAL_ERROR:
      {
	out << "fatal error";
	break;
      }
    case IO_EXCEPTION:
      {
	out << "IO exception";
	break;
      }
    case SYNTAX_EXCEPTION:
      {
	out << "syntax exception";
	break;
      }
    case NOT_FOUND_EXCEPTION:
      {
	out << "item not found";
	break;
      }
    case DUPLICATE_EXCEPTION:
      {
	out << "item duplicate";
	break;
      }
    }

  out << endl << "\tDescription: " << this->get_description() << endl;

  return;
}


string FileParserException::get_description() const
{
  return m_description;
}

int FileParserException::get_type() const
{
  return m_type;
}
