// Copyright (C) 1999 Jean-Marc Valin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "VNSum.h"
#include "net_types.h"
#include "Object.h"
#include "Vector.h"

//DECLARE_NODE(VNSum)
NODE_INFO(VNSum,"Signal:DSP", "INPUT", "OUTPUT", "")

VNSum::VNSum(string nodeName, ParameterSet params) 
   : Node(nodeName, params)
   , accum_count()
{
   outputID = addOutput("OUTPUT");
   inputID = addInput("INPUT");
}

void VNSum::specificInitialize()
{
   this->Node::specificInitialize();
   sum = ObjectRef (new Vector<float>);
   nsum = ObjectRef (new Vector<float>);
   accum_count.resize(0);
   naccum_count.resize(0);
}

void VNSum::reset()
{
   this->Node::reset();
   sum = ObjectRef (new Vector<float>);
   nsum = ObjectRef (new Vector<float>);
   accum_count.resize(0);
   naccum_count.resize(0);
}

ObjectRef VNSum::getOutput(int output_id, int count)
{
   if (output_id==outputID)
   {
      for (int i=processCount+1 ; i<=count ; i++ )
      {
         NodeInput input = inputs[inputID];
         ObjectRef inputResult = input.node->getOutput(input.outputID, i);
         if (!inputResult->status)
         {
            //cerr << "VNSum received: " << inputResult << endl;
            Vector<ObjectRef> &in = object_cast<Vector<ObjectRef> > (inputResult);
            int length = in.size();
            if (length > object_cast<Vector<float> > (sum).size())
            {
               object_cast<Vector<float> > (sum).resize(length,0);
               object_cast<Vector<float> > (nsum).resize(length,0);
               accum_count.resize(length,0);
               naccum_count.resize(length,0);
            }
            for (int j=0;j<length;j++)
            {
               if (in[j]->status == Object::valid)
               {
                  accum_count[j]++;
                  object_cast<Vector<float> > (sum)[j] += dereference_cast<float> (in[j]);
                  //cerr << dereference_cast<float> (in[j]) << " > " 
                  //     << object_cast<Vector<float> > (sum)[j]/accum_count[j]
                  //     << " ? " << endl;
                  if (dereference_cast<float> (in[j]) < object_cast<Vector<float> > (sum)[j]/float(accum_count[j]))
                  {
                     naccum_count[j]++;
                     float fact = 1/float(naccum_count[j]);
                     object_cast<Vector<float> > (nsum)[j] = (1-fact)*object_cast<Vector<float> > (nsum)[j]
                     + fact*dereference_cast<float> (in[j]);
                  }
               }
            }
            //cerr << "sum = " << sum << endl;
         }
         processCount = i;
      }
      return nsum;
   }
   else 
      throw new NodeException (this, "VNSum: Unknown output id", __FILE__, __LINE__);
}
